@@ -116,7 +116,7 @@ EOT
           failure.inspected_expected = AssertionMessage.convert(expected)
           failure.inspected_actual = AssertionMessage.convert(actual)
           failure.user_message = message
-          raise
+          raise failure # For JRuby. :<
         end
       end
 
@@ -1019,7 +1019,7 @@ EOT
                                        expected, actual,
                                        expected, actual)
           assert_block(full_message) do
-            expected.send(operator, actual)
+            expected.__send__(operator, actual)
           end
         end
       end
@@ -1129,7 +1129,7 @@ EOT
       def assert_predicate(object, predicate, message=nil)
         _wrap_assertion do
           assert_respond_to(object, predicate, message)
-          actual = object.send(predicate)
+          actual = object.__send__(predicate)
           full_message = build_message(message,
                                        "<?>.? is true value expected but was\n" +
                                        "<?>",
@@ -1151,7 +1151,7 @@ EOT
       def assert_not_predicate(object, predicate, message=nil)
         _wrap_assertion do
           assert_respond_to(object, predicate, message)
-          actual = object.send(predicate)
+          actual = object.__send__(predicate)
           full_message = build_message(message,
                                        "<?>.? is false value expected but was\n" +
                                        "<?>",
@@ -1431,6 +1431,8 @@ EOT
 
           MAX_DIFF_TARGET_STRING_SIZE = 1000
           def max_diff_target_string_size
+            return @@max_diff_target_string_size if @@max_diff_target_string_size
+
             size = ENV["TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE"]
             if size
               begin
@@ -1442,6 +1444,11 @@ EOT
             size || MAX_DIFF_TARGET_STRING_SIZE
           end
 
+          @@max_diff_target_string_size = nil
+          def max_diff_target_string_size=(size)
+            @@max_diff_target_string_size = size
+          end
+
           def diff_target_string?(string)
             if string.respond_to?(:bytesize)
               string.bytesize < max_diff_target_string_size
@@ -1525,8 +1532,20 @@ EOM
         end
 
         class Inspector
-          def initialize(object)
+          include Comparable
+
+          class << self
+            def cached_new(object, inspected_objects)
+              inspected_objects[object.object_id] ||=
+                new(object, inspected_objects)
+            end
+          end
+
+          attr_reader :object
+          def initialize(object, inspected_objects={})
+            @inspected_objects = inspected_objects
             @object = object
+            @inspected_objects[@object.object_id] = self
             @inspect_target = inspect_target
           end
 
@@ -1543,12 +1562,20 @@ EOM
             @inspect_target.pretty_print_cycle(q)
           end
 
+          def <=>(other)
+            if other.is_a?(self.class)
+              @object <=> other.object
+            else
+              @object <=> other
+            end
+          end
+
           private
           def inspect_target
             if HashInspector.target?(@object)
-              HashInspector.new(@object)
+              HashInspector.new(@object, @inspected_objects)
             elsif ArrayInspector.target?(@object)
-              ArrayInspector.new(@object)
+              ArrayInspector.new(@object, @inspected_objects)
             else
               @object
             end
@@ -1562,8 +1589,14 @@ EOM
             end
           end
 
-          def initialize(hash)
-            @hash = hash
+          def initialize(hash, inspected_objects)
+            @inspected_objects = inspected_objects
+            @hash = {}
+            hash.each do |key, value|
+              key = Inspector.cached_new(key, @inspected_objects)
+              value = Inspector.cached_new(value, @inspected_objects)
+              @hash[key] = value
+            end
           end
 
           def inspect
@@ -1596,8 +1629,7 @@ EOM
             rescue ArgumentError
             end
             keys.each do |key|
-              yield(Inspector.new(key),
-                    Inspector.new(@hash[key]))
+              yield(key, @hash[key])
             end
           end
         end
@@ -1609,8 +1641,11 @@ EOM
             end
           end
 
-          def initialize(array)
-            @array = array
+          def initialize(array, inspected_objects)
+            @inspected_objects = inspected_objects
+            @array = array.collect do |element|
+              Inspector.cached_new(element, @inspected_objects)
+            end
           end
 
           def inspect
@@ -1629,10 +1664,8 @@ EOM
             @array.pretty_print_cycle(q)
           end
 
-          def each
-            @array.each do |element|
-              yield(Inspector.new(element))
-            end
+          def each(&block)
+            @array.each(&block)
           end
         end
 
@@ -1743,7 +1776,7 @@ EOM
           end
 
           def method_missing(name, *args, &block)
-            @exception.send(name, *args, &block)
+            @exception.__send__(name, *args, &block)
           end
 
           private
@@ -1799,10 +1832,10 @@ EOM
             elsif exception_type.is_a?(Exception)
               exception_objects << exception_type
             else
-              @test_case.send(:assert,
-                              Exception >= exception_type,
-                              "Should expect a class of exception, " +
-                              "#{exception_type}")
+              @test_case.__send__(:assert,
+                                  Exception >= exception_type,
+                                  "Should expect a class of exception, " +
+                                  "#{exception_type}")
               exception_classes << exception_type
             end
           end
@@ -1811,7 +1844,7 @@ EOM
 
         def expected_class?(actual_exception, equality)
           @expected_classes.any? do |expected_class|
-            actual_exception.send(equality, expected_class)
+            actual_exception.__send__(equality, expected_class)
           end
         end
 
